import os

import curl
import curl.nn as nn
import torch


class LeNet5(nn.Module):
    def __init__(self):
        super().__init__()
        self.feature = nn.Sequential(
            # 1
            nn.Conv2d(
                in_channels=1, out_channels=6, kernel_size=5, stride=1, padding=2
            ),  # 28*28->32*32-->28*28
            nn.ReLU(),
            nn.AvgPool2d(kernel_size=2, stride=2),  # 14*14
            # 2
            nn.Conv2d(in_channels=6, out_channels=16, kernel_size=5, stride=1),  # 10*10
            nn.ReLU(),
            nn.AvgPool2d(kernel_size=2, stride=2),  # 5*5
        )
        self.classifier = nn.Sequential(
            nn.Flatten(),
            nn.Linear(in_features=16 * 5 * 5, out_features=120),
            nn.ReLU(),
            nn.Linear(in_features=120, out_features=84),
            nn.ReLU(),
            nn.Linear(in_features=84, out_features=10),
        )

    def forward(self, x):
        return self.classifier(self.feature(x))


if __name__ == "__main__":
    curl.init()
    model = LeNet5()
    model.load_state_dict(
        torch.load(os.path.join(os.path.dirname(__file__), "lenet5.pth"))
    )
    inputs = curl.cryptensor(torch.rand(1, 1, 28, 28))
    encrypted_model = model.encrypt(src=0)
    result = model.forward(inputs)
    print("Result: ", result)

# Use DefaultLUTConfig as LUTConfig from lut_config.py
from curl.config.lut_config import DefaultLUTConfig

LUTConfig = DefaultLUTConfig
